#!/bin/bash

PROGNAME=$(basename $0)
VERSION="1.0.0"

function error_exit 
{
	#####	
	# 	Function for exit due to fatal program error
	# 	Accepts 1 argument
	#		string containing descriptive error message
	#####

	local err_msg
	
	err_msg="${PROGNAME}: ${1}"
	echo ${err_msg} >&2
	exit 1
}


function usage
{
	#####
	#	Function to display usage message (does not exit)
	#	No arguments
	#####

	echo "Usage:"
	echo "	${PROGNAME}  i <server name / ip address> -p <port> -u <externally visible address> -c <streaming server clustersize>"
	echo "	${PROGNAME} -h | --help"
}


function helptext
{
	#####
	#	Function to display help message for program
	#	No arguments
	#####
	
	local tab=$(echo -en "\t\t")
		
	cat <<- -EOF-
	
	${PROGNAME} ver. ${VERSION}	
	This program configures the linux installation of Dimdim Web Conference Server.
	
	$(usage)
	
	Options:
	
	-i server name / ip address
			This is the ip address on the machine that the web server will bind to.
	-p port number	This the port number of the http server
	-u externally visible server address
			This is the http server address that is accessible to all the
			people who are expected to attend conferences on this server.
	-c streaming server cluster size
			Number of streaming servers used to support the conference server.
			
	NOTE: You must be the superuser to run this script.
		
	-EOF-
}


function root_check
{
	#####
	#	Function to check if user is root
	#	No arguments
	#####
	
	if [ "$(id | sed 's/uid=\([0-9]*\).*/\1/')" != "0" ]; then
		error_exit "You must be the superuser to run this script."
	fi
}

umask 066

if [ "$1" = "--help" ]; then
	helptext
	exit 1
fi


BIA=
PN=
SN=
NSS=

while getopts ":i:p:u:c:h" opt; do
	case $opt in
		i )	BIA=${OPTARG}
			;;
		p )	PN=${OPTARG}
			;;
		u )	SN=${OPTARG}
			;;
		c )	NSS=${OPTARG}
			;;
		h )	helptext
			exit 1
			;;
		* )	usage
			exit 1
			;;
	esac
done

# Version must be set

if [ "$JAVA_HOME" = "" ]; then
	error_exit "Dimdim Web Conference Server requires JAVA_HOME environment variable to be set."
fi

if [ "$BIA" = "" ]; then
	error_exit "Dimdim Web Conference Server requires bind ip address to be specified."
fi

if [ "$PN" = "" ]; then
	error_exit "Dimdim Web Conference Server requires port number to be specified."
fi

if [ "$SN" = "" ]; then
	error_exit "Dimdim Web Conference Server requires external server address to be specified."
fi

if [ "$NSS" = "" ]; then
	error_exit "Dimdim Web Conference Server requires number of streaming servers to be specified."
fi


echo "Configuring the Conference Server and Streaming Server Cluster"
echo ""

if [ "$PN" -lt "1024" ]; then
	root_check
fi

./updateFiles.bash $BIA $PN $SN $NSS


