#!/bin/bash
#

BIND_IP_ADDRESS=$1
PORT_NUMBER=$2
SERVER_NAME=$3
NUM_STREAMING_SERVERS=$4

echo "Bind IP Address: $BIND_IP_ADDRESS"
echo "Port Number: $PORT_NUMBER"
echo "External Server Name: $SERVER_NAME"
echo "Number of Streaming Servers: $NUM_STREAMING_SERVERS"

INSTALL_DIR=`pwd`
echo "Installation Directory: $INSTALL_DIR"

echo "-------------------------------"


JAVA_HOME_SAFE=`echo $JAVA_HOME | sed "s/\//DIMDIM_SLASH/g"`
JAVA_COMMAND_SAFE="$JAVA_HOME_SAFE/bin/java"
#
SS_PORT=30000
#echo $SS_PORT
#
TOMCAT_DIR="$INSTALL_DIR/ConferenceServer/apache-tomcat-5.5.17"
#echo $TOMCAT_DIR
#
DIMDIM_WEBAPP_DIR="$TOMCAT_DIR/webapps/dimdim"
DIMDIM_WEBAPP_DIR_SAFE=`echo $DIMDIM_WEBAPP_DIR | sed "s/\//DIMDIM_SLASH/g"`
#echo $DIMDIM_WEBAPP_DIR
#
CONF_PROPS_FILE="$DIMDIM_WEBAPP_DIR/WEB-INF/classes/resources/DimDimConference.properties"
CONF_PROPS_FILE_BAK="$CONF_PROPS_FILE.bak"
#echo $CONF_PROPS_FILE
#
ROOT_WEBAPP_DIR="$TOMCAT_DIR/webapps/ROOT"
#echo $ROOT_WEBAPP_DIR
#
ROOT_RED5CONF_PROPS_FILE="$ROOT_WEBAPP_DIR/WEB-INF/classes/resources/Red5Config.properties"
#echo $ROOT_RED5CONF_PROPS_FILE
#
SERVER_CONF_FILE="$TOMCAT_DIR/conf/server.xml"
SERVER_CONF_FILE_BAK="$TOMCAT_DIR/conf/server.xml.bak"
#echo $SERVER_CONF_FILE
#
WRAPPER_CONF_FILE="$TOMCAT_DIR/conf/wrapper.conf"
WRAPPER_CONF_FILE_BAK="$TOMCAT_DIR/conf/wrapper.conf.bak"
#echo $WRAPPER_CONF_FILE
#
STREAMING_CLUSTER_DIR="$INSTALL_DIR/Streaming Server Cluster"
#echo $STREAMING_CLUSTER_DIR
#
STREAMING_SERVER_RED5_PROPS_FILE="conf/red5.properties"
#echo $STREAMING_SERVER_RED5_PROPS_FILE
#
STREAMING_SERVER_WEBAPP_PROPS_FILE="webapps/dimdimPublisher/WEB-INF/red5-web.properties"
#echo $STREAMING_SERVER_WEBAPP_PROPS_FILE

##
##
##
##	Streaming Servers Replication and properties setup
##	Write the Red 5 Properties file in the root webapp
##
##

echo "Writing root webapp red 5 config properties file"

echo "dimdim.red5.config.count=$NUM_STREAMING_SERVERS" > $ROOT_RED5CONF_PROPS_FILE
echo "" >> $ROOT_RED5CONF_PROPS_FILE
echo "dimdim.red5.config.1.directory=$STREAMING_CLUSTER_DIR/server1" >> $ROOT_RED5CONF_PROPS_FILE
echo "dimdim.red5.config.1.port=$SS_PORT" >> $ROOT_RED5CONF_PROPS_FILE
echo "dimdim.red5.config.1.app=dimdimPublisher" >> $ROOT_RED5CONF_PROPS_FILE
echo "" >> $ROOT_RED5CONF_PROPS_FILE

STREAMING_SERVER_DIR="$STREAMING_CLUSTER_DIR/server1/"
SS_FILE_1="$STREAMING_SERVER_DIR/$STREAMING_SERVER_RED5_PROPS_FILE"
SS_FILE_2="$STREAMING_SERVER_DIR/$STREAMING_SERVER_WEBAPP_PROPS_FILE"

echo "rtmp.host_port=0.0.0.0:1935" > "$SS_FILE_1"
echo "http.host=0.0.0.0" >> "$SS_FILE_1"
echo "http.port=5080" >> "$SS_FILE_1"
echo "rtmpt.host=0.0.0.0" >> "$SS_FILE_1"
echo "rtmpt.port=$SS_PORT" >> "$SS_FILE_1"
echo "debug_proxy.host_port=0.0.0.0:1936" >> "$SS_FILE_1"
echo "proxy_forward.host_port=127.0.0.1:5080" >> "$SS_FILE_1"

echo "webapp.contextPath=/dimdimPublisher" > "$SS_FILE_2"
echo "webapp.virtualHosts=*, localhost, localhost:$SS_PORT, 127.0.0.1:$SS_PORT" >> "$SS_FILE_2"

#cat "$SS_FILE_1"
#echo "----------"
#cat "$SS_FILE_2"
#echo "----------"

CURRENT_DIR=`pwd`
#echo $CURRENT_DIR

cd "$STREAMING_CLUSTER_DIR"

STREAMING_SERVER_NUMBER=1
while [ $STREAMING_SERVER_NUMBER -lt $NUM_STREAMING_SERVERS ]; do
  SERVER_NUM=$((STREAMING_SERVER_NUMBER+1))
  SS_PORT=$((SS_PORT+1))

	echo "dimdim.red5.config.$SERVER_NUM.directory=$STREAMING_CLUSTER_DIR/server$SERVER_NUM" >> $ROOT_RED5CONF_PROPS_FILE
	echo "dimdim.red5.config.$SERVER_NUM.port=$SS_PORT" >> $ROOT_RED5CONF_PROPS_FILE
	echo "dimdim.red5.config.$SERVER_NUM.app=dimdimPublisher" >> $ROOT_RED5CONF_PROPS_FILE
	echo "" >> $ROOT_RED5CONF_PROPS_FILE

  cp -r server1 "server$SERVER_NUM"

  STREAMING_SERVER_DIR="$STREAMING_CLUSTER_DIR/server$SERVER_NUM"
  SS_FILE_1="$STREAMING_SERVER_DIR/$STREAMING_SERVER_RED5_PROPS_FILE"
  SS_FILE_2="$STREAMING_SERVER_DIR/$STREAMING_SERVER_WEBAPP_PROPS_FILE"
  
#  echo $STREAMING_SERVER_DIR
#  echo $SS_FILE_1
#  echo $SS_FILE_2

	echo "rtmp.host_port=0.0.0.0:1935" > "$SS_FILE_1"
	echo "http.host=0.0.0.0" >> "$SS_FILE_1"
	echo "http.port=5080" >> "$SS_FILE_1"
	echo "rtmpt.host=0.0.0.0" >> "$SS_FILE_1"
	echo "rtmpt.port=$SS_PORT" >> "$SS_FILE_1"
	echo "debug_proxy.host_port=0.0.0.0:1936" >> "$SS_FILE_1"
	echo "proxy_forward.host_port=127.0.0.1:5080" >> "$SS_FILE_1"

	echo "webapp.contextPath=/dimdimPublisher" > "$SS_FILE_2"
	echo "webapp.virtualHosts=*, localhost, localhost:$SS_PORT, 127.0.0.1:$SS_PORT" >> "$SS_FILE_2"

  STREAMING_SERVER_NUMBER=$((STREAMING_SERVER_NUMBER+1))
done
echo "Steaming Servers Configuration Complete"

cd $CURRENT_DIR

if [ ! -f "$CONF_PROPS_FILE_BAK" ]; then
	cp  "$CONF_PROPS_FILE" "$CONF_PROPS_FILE_BAK" 
fi

if [ ! -f "$WRAPPER_CONF_FILE_BAK" ]; then
	cp  "$WRAPPER_CONF_FILE" "$WRAPPER_CONF_FILE_BAK" 
fi

##
##
##	Conference Server Properties files token replacements
##
##

#echo $DIMDIM_WEBAPP_DIR_SAFE
#D2=`echo $DIMDIM_WEBAPP_DIR_SAFE | sed "s/DIMDIM_SLASH/\//g"`
#echo $D2

cp "$CONF_PROPS_FILE" "$CONF_PROPS_FILE.orig"
#
# Clean out old lines
#
sed "/dimdim.serverAddress/d" < "$CONF_PROPS_FILE" > "$CONF_PROPS_FILE.new"
mv "$CONF_PROPS_FILE.new" "$CONF_PROPS_FILE"
sed "/dimdim.serverPortNumber/d" < "$CONF_PROPS_FILE" > "$CONF_PROPS_FILE.new"
mv "$CONF_PROPS_FILE.new" "$CONF_PROPS_FILE"
#
# Add new lines for the server address and port number
#
sed "/dimdim.webappLocalPath/a dimdim.serverAddress=$SERVER_NAME" < "$CONF_PROPS_FILE" > "$CONF_PROPS_FILE.new"
mv "$CONF_PROPS_FILE.new" "$CONF_PROPS_FILE"
sed "/dimdim.webappLocalPath/a dimdim.serverPortNumber=$PORT_NUMBER" < "$CONF_PROPS_FILE" > "$CONF_PROPS_FILE.new"
mv "$CONF_PROPS_FILE.new" "$CONF_PROPS_FILE"
#
# Setup the webapp local path
#
sed "/dimdim.webappLocalPath/d" < "$CONF_PROPS_FILE" > "$CONF_PROPS_FILE.new"
mv "$CONF_PROPS_FILE.new" "$CONF_PROPS_FILE"
sed "/dimdim.serverPortNumber/a dimdim.webappLocalPath=$DIMDIM_WEBAPP_DIR_SAFE" < "$CONF_PROPS_FILE" > "$CONF_PROPS_FILE.new"
mv "$CONF_PROPS_FILE.new" "$CONF_PROPS_FILE"
#
sed "s/DIMDIM_SLASH/\//g" < "$CONF_PROPS_FILE" > "$CONF_PROPS_FILE.new"
mv "$CONF_PROPS_FILE.new" "$CONF_PROPS_FILE"
#
# Setup the email server defaults. This pattern replacement will not
# affect any changes user may have done to the file manually.
#
sed "s/EmailServer/smtp.atlarge.net/g" < "$CONF_PROPS_FILE" > "$CONF_PROPS_FILE.new"
mv "$CONF_PROPS_FILE.new" "$CONF_PROPS_FILE"
sed "s/EmailUser/invitation.communiva/g" < "$CONF_PROPS_FILE" > "$CONF_PROPS_FILE.new"
mv "$CONF_PROPS_FILE.new" "$CONF_PROPS_FILE"
sed "s/EmailPassword/summer2006/g" < "$CONF_PROPS_FILE" > "$CONF_PROPS_FILE.new"
mv "$CONF_PROPS_FILE.new" "$CONF_PROPS_FILE"
sed "s/EmailSender/invitations\@dimdim.com/g" < "$CONF_PROPS_FILE" > "$CONF_PROPS_FILE.new"
mv "$CONF_PROPS_FILE.new" "$CONF_PROPS_FILE"
sed "s/EmailPort/25/g" < "$CONF_PROPS_FILE" > "$CONF_PROPS_FILE.new"
mv "$CONF_PROPS_FILE.new" "$CONF_PROPS_FILE"
dos2unix "$CONF_PROPS_FILE" >&! /dev/null

echo "Conference Properties File setup complete"

cp "$WRAPPER_CONF_FILE" "$WRAPPER_CONF_FILE.orig"
#
sed "/wrapper.java.command/a DIMDIM_WRAPPER_JAVA_COMMAND=$JAVA_COMMAND_SAFE" < "$WRAPPER_CONF_FILE" > "$WRAPPER_CONF_FILE.new"
mv "$WRAPPER_CONF_FILE.new" "$WRAPPER_CONF_FILE"
#
sed "/wrapper.java.command/d" < "$WRAPPER_CONF_FILE" > "$WRAPPER_CONF_FILE.new"
mv "$WRAPPER_CONF_FILE.new" "$WRAPPER_CONF_FILE"
#
sed "s/DIMDIM_WRAPPER_JAVA_COMMAND/wrapper.java.command/g" < "$WRAPPER_CONF_FILE" > "$WRAPPER_CONF_FILE.new"
mv "$WRAPPER_CONF_FILE.new" "$WRAPPER_CONF_FILE"
#
sed "s/DIMDIM_SLASH/\//g" < "$WRAPPER_CONF_FILE" > "$WRAPPER_CONF_FILE.new"
mv "$WRAPPER_CONF_FILE.new" "$WRAPPER_CONF_FILE"
dos2unix  "$WRAPPER_CONF_FILE" >&! /dev/null
#
if [ ! -f "$SERVER_CONF_FILE_BAK" ]; then
  cp "$SERVER_CONF_FILE" "$SERVER_CONF_FILE_BAK"
else
  cp "$SERVER_CONF_FILE_BAK" "$SERVER_CONF_FILE" 
fi
#
cp $SERVER_CONF_FILE "$SERVER_CONF_FILE.orig"
if [ "$BIND_IP_ADDRESS" = "ALL" ] ; then
	sed "s/DIMDIM_PORT_NUMBER/$PORT_NUMBER/g" < "$SERVER_CONF_FILE" > "$SERVER_CONF_FILE.new"
else
	sed "s/DIMDIM_PORT_NUMBER/$PORT_NUMBER\" address=\"$BIND_IP_ADDRESS/g" < "$SERVER_CONF_FILE" > "$SERVER_CONF_FILE.new"
fi
mv "$SERVER_CONF_FILE.new" "$SERVER_CONF_FILE"
dos2unix "$SERVER_CONF_FILE" >&! /dev/null

echo "Server Configuration Files setup complete"

## -----------------------------------------------------------------------------------

cd "$TOMCAT_DIR/bin"
chmod +x *
dos2unix dimdim

cd $CURRENT_DIR

if [ ! -f dimdim ]; then
  ln -s "$TOMCAT_DIR/bin/dimdim" .
  ln -s "$TOMCAT_DIR/logs/wrapper.log" .
  ln -s "$CONF_PROPS_FILE" .
  ln -s "$SERVER_CONF_FILE" .
  ln -s "$WRAPPER_CONF_FILE" .
fi

#echo "----------"
#cat $ROOT_RED5CONF_PROPS_FILE
#echo "----------"
#cat "$SS_FILE_1"
#echo "----------"
#cat "$SS_FILE_2"
#echo "----------"
#cat $CONF_PROPS_FILE

